@@ -7,7 +7,8 @@ from chameleon.namespaces import I18N_NS
 from chameleon.namespaces import TAL_NS
 from chameleon.program import ElementProgram
 from chameleon.zpt.program import MacroProgram
-
+from chameleon.tal import parse_defines
+from chameleon.utils import decode_htmlentities
 
 from .python import _extract_python
 from . import register_extractor
@@ -79,6 +80,11 @@ class Extractor(ElementProgram):
             sys.exit(1)
         super(Extractor, self).__init__(source, filename=filename)
 
+    def visit(self, kind, args):
+        visitor = getattr(self, 'visit_%s' % kind, None)
+        if visitor is not None:
+            return visitor(*args)
+
     def visit_element(self, start, end, children):
         if self.translatestack and self.translatestack[-1]:
             self.translatestack[-1].add_element(start)
@@ -119,7 +125,7 @@ class Extractor(ElementProgram):
 
             for (attribute, value) in attributes.items():
                 for source in self.get_code_for_attribute(attribute, value):
-                    self.parse_python(source)
+                    self.parse_python(decode_htmlentities(source))
 
         for child in children:
             self.visit(*child)
@@ -149,7 +155,10 @@ class Extractor(ElementProgram):
         if attribute[0] == TAL_NS:
             if attribute[1] in ['content', 'replace']:
                 yield value
-            elif attribute[2] in ['define', 'repeat']:
+            if attribute[1] == 'define':
+                for (scope, var, value) in parse_defines(value):
+                    yield value
+            elif attribute[1] == 'repeat':
                 yield value.split(None, 1)[1]
         else:
             for source in EXPRESSION.findall(value):
@@ -158,9 +167,9 @@ class Extractor(ElementProgram):
     def parse_python(self, source):
         if not isinstance(source, bytes):
             source = source.encode('utf-8')
-        for message in _extract_python(self.filename, source, self.options):
+        for message in _extract_python(self.filename, source, self.options, self.linenumber):
             self.messages.append(Message(*message[:6],
-                location=(self.filename, self.linenumber)))
+                location=(self.filename, self.linenumber + message.location[1])))
 
 
 @register_extractor('xml', ['.pt', '.zpt'])
